<?php

namespace Drupal\flockler\Plugin\Block;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Html;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'FlocklerBlock' block.
 *
 * @Block(
 *  id = "flockler_block",
 *  admin_label = @Translation("Flockler block"),
 * )
 */
class FlocklerBlock extends BlockBase {

  /**
   * Guzzle Http Client.
   *
   * @var GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Feed URL'),
      '#description' => $this->t('The URL of the Flockler feed'),
      '#default_value' => $this->configuration['url'],
      '#maxlength' => 64,
      '#required' => TRUE,
      '#size' => 64,
      '#weight' => '0',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['url'] = $form_state->getValue('url');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $build['content'] = $this->fetchFeed();
    $build['#attributes'] = [
      'class' => [
        'flockler-articles',
      ],
    ];
    return $build;
  }

  private function fetchFeed() {
    $url = $this->configuration['url'];

    // TODO: use dependency injection.
    $response = \Drupal::httpClient()->get($url);
    $body = $response->getBody()->__toString();
    $jsonObject = Json::decode($body, true);

    // TODO: cache contexts.
    return $this->formatFeed($jsonObject);
  }

  private function formatFeed($jsonObject) {
    $render = [];

    if (!empty($jsonObject)) {
      $render = [
        '#attributes' => [
          'class' => [
            'flockler-articles',
          ],
        ],
      ];

      foreach ($jsonObject['articles'] as $article) {
        $render[] = $this->formatArticle($article);
      }
    }
    return $render;
  }

  /**
   * Prepare an article
   *
   * @param array $article
   *   The JSON representation of the article in the feed.
   *
   * @return array
   *   A render array.
   */
  private function formatArticle($article) {
    $render = [];

    switch ($article['type']) {
      case 'tweet':
        $render = $this->formatTweet($article);
        break;
      case 'instagram':
        $render = $this->formatInstagram($article);
        break;
    }

    return $render;
  }

  /**
   * Prepare a render array for a tweet.
   *
   * @param array $article
   *   The JSON representation of the tweet.
   *
   * @return array
   *   A render array for the tweet.
   */
  private function formatTweet($article) {

    $tweet = $article['attachments']['tweet'];

    $render = [
      '#theme' => 'flockler_article',
      '#content' => [
        'type' => 'tweet',
        'profile_image' => $tweet['profile_image_url'],
        'profile_name' => $tweet['name'],
        'profile_link' => 'https://twitter.com/' . $tweet['screen_name'],
        'text' => $tweet['text'],
        'image' => $tweet['media_url'],
        'link' => 'https://twitter.com/' . $tweet['screen_name'] . '/status/' . $tweet['tweet_id_str'],
      ],
    ];

    if (!$this->isImage($tweet['media_url'])) {
      $videoPreview = $this->getVideoThumbnail($tweet['media_url']);
      if ($videoPreview) {
        $render['#content']['image'] = $videoPreview;
      }
    }

    return $render;
  }

  /**
   * Prepare a render array for an instagram post.
   *
   * @param array $article
   *   The JSON representation of the post.
   *
   * @return array
   *   A render array for the post.
   */
  private function formatInstagram($article) {
    $post = $article['attachments']['instagram_item'];

    $render = [
      '#theme' => 'flockler_article',
      '#content' => [
        'type' => 'instagram',
        'text' => $post['caption'],
        'profile_image' => $post['profile_picture'],
        'profile_name' => $post['user_full_name'],
        'profile_link' => 'https://www.instagram.com/' . $post['username'],
        'link' => $post['link'],
        'image' => $article['cover_url'],
      ],
    ];
    return $render;
  }

  /**
   * Check if a URL refers to an image.
   *
   * @param string $url
   *   The media URL to check.
   *
   * @return bool
   *   TRUE if the URL refers to an image.
   */
  private function isImage($url) {
    $urlParts = explode('.', $url);
    $extension = end($urlParts);

    $imageTypes = [
      'jpg',
      'jpeg',
      'png',
      'gif',
      'svg',
    ];

    return in_array(strtolower($extension), $imageTypes);
  }

  /**
   * Get the thumbnail for a video.
   *
   * @param string $url
   *   The media URL from the article.
   *
   * @return string
   *   The URL of the video thumbnail.
   */
  private function getVideoThumbnail($url) {
    $imageUrl = '';

    if (!empty($url)) {
      // Fetch the page.
      $response = \Drupal::httpClient()->get($url);
      $body = $response->getBody()->__toString();

      // Does the page have the twitter:image metatag? If so, use it.
      $html = Html::load($body);
      $metas = $html->getElementsByTagName('meta');
      foreach ($metas as $meta) {
        $name = $meta->getAttribute('name');
        if ($name == 'twitter:image') {
          $imageUrl = $meta->getAttribute('content');;
        }
      }
    }

    return $imageUrl;
  }
}
